#!/usr/bin/perl
###########################################################################################################
# This script allows for any application to pass a queue of files to be restored or archived to PresSTORE #
# Written by Mike Szumlinski																			  #
# szumlins@mac.com																						  #
# August 22, 2011																						  #
#																										  #	
# Special thanks to André Aulich for the paradigm used in this script									  #
###########################################################################################################

#############################
# Known Issues/Future Plans #
#############################
# Currently does not support preview linking to PresSTORE. -> moved to external scripts
#
# Currently does not support metadata export from external sources. -> moved to external scripts
#
# Does not validate that files being restored have been removed from the archive queue - will cause 
# issues if a file is requested for restore before the chosen archive interval is run
#



##################
# User variables #
##################

# As of 8-2011, this script supports sockets, so aw-queue.pl does not have to be run on the PresSTORE server locally
#
# If running on the PresSTORE server, leave this as localhost.  Otherwise, input a valid DNS name of PresSTORE server
# PresSTORE server uses DNS for all remote connectivity, so proper forward/reverse DNS on machine running this script is necessary
$hostname = "localhost";
# Username of a user on the PresSTORE server with permissions to archive and restore
$username = "<someuser>";
# Password of a user on the PresSTORE server with permissions to archive and restore
$password = "<somepass>";
# Default nsdchat port is 9001.  Do not change unless you specifically changed this in your PresSTORE server config
$port = "9001";

#this is the path for the original media queue to be archived/restored
$full_queue_path = $ARGV[0];

#this is the method being used (archive or restore)
$method = $ARGV[1];

#generate a uique session ID based on the date
$sessionid=`date | md5`;
$sessionid=~ s/\n//;

#this is the path to nsdchat.  You need to change this if you do not have a standard PresSTORE install
$nsdchat = "/usr/local/aw/bin/nsdchat -s awsock:/$username:$password:$sessionid\@$hostname:$port -c";

#if the filesystem for the archived content lives on a different PresSTORE client than localhost, change that here
$client = "localhost";

#index ID - this is passed as the third argument to the CLI
$index = $ARGV[3];

#log file location.  Change this if you want this going someplace different.  Must change on non-MacOS X systems
$logfile = "/Library/Logs/aw-queue.log";


#make sure the input is good

#if cli variables are empty, print usage to user/log

if ($ARGV[0] eq '' || $ARGV[1] eq '' || $ARGV[2] eq ''){
	print "\nusage: aw-queue.pl queue_file archive|restore archive_plan archive_index\n\n";
	print "queue_file: full path to queue file.\n";
	print "archive|restore: select whether to archive or restore from archive the source_file.\n";
	print "archive_plan: which archive plan you want PresSTORE to use.\n";
	print "archive_index: only necessary for restore, this is the PresSTORE index we will look up our files from.\n\n";
	exit
}

#if method is not proper, inform user

if ($ARGV[1] ne 'archive' && $ARGV[1] ne 'restore'){
	print "\nMethod $ARGV[1] not defined.  Please choose \"archive\" or \"restore\"\n\n";
	exit
}

#try a simple login to the server to validate nsdchat
$testlogin = `$nsdchat srvinfo hostname`;
$testlogin =~ s/\n//;
if ($testlogin ne $hostname){
	$date = `date`;
	$date =~ s/\n//;
	print "\nCould not connect to PresSTORE socket using \"$nsdchat\".  Please verify variables.\n";	
	exit		
}

#check that the archive plan chosen is valid and enabled

if ($ARGV[2] eq ''){
	print "\narchive_plan not defined\n\n";
} else {
	$plan_check = `$nsdchat ArchivePlan $ARGV[2] enabled`;
	if($plan_check == '0'){
		print "\nArchive plan $ARGV[2] disabled.  Please enable plan\n\n";
		exit
	}
	if($plan_check != '1'){
		print "\nArchive plan $ARGV[2] not found.\n\n";
		exit
	}
}

#ok, now that we know we have all the information we need, lets open our log file for writing in case there are errors in the process
#if the log file doesn't exist, we create it, otherwise we append

if(-e $logfile){
	open LOGFILE, ">>", $logfile or die $!;
} else {
	open LOGFILE, ">", $logfile or die $!;
}



#check to see if our queue file is actually there.  if it isn't, print to the log and quit.
if(! -e $full_queue_path){
	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Queue file $full_queue_path does not exist! Exiting script, please check your queue file.\n";	
	exit
}

if(! -s $full_queue_path){
	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Queue file $full_queue_path is empty. Nothing to do\n";	
	exit
}
	

#run archive method
if($method eq 'archive'){
	#create job
	$archive_selection = `$nsdchat ArchiveSelection create $client $ARGV[2]`;
	$archive_selection =~ s/\n//;
	
	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Archive selection $archive_selection created successfully\n";
	
	#open queue file, add each file to archive queue
	open FILE, "<",$full_queue_path or die $!;

	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Archive queue file $full_queue_path successfully opened\n";

	$i = 0;
	while (<FILE>){
		$_ =~ s/\n//;
		@handles[$i] = `$nsdchat ArchiveSelection $archive_selection addentry {$_}`;

		$date = `date`;
		$date =~ s/\n//;
		print LOGFILE "$date -- File $_ generated archive handle @handles[$i]";		
		
		$i++;
	}
		
	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Preparing $i files for archive\n";		

	#submit archive job to run
	$job_id = `$nsdchat ArchiveSelection $archive_selection submit yes`;
	$job_id =~ s/\n//;
	
	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Archive job $job_id successfully submitted\n";

	open FILE,">",$full_queue_path;
	close FILE;
	
	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Archive queue emptied\n";

}

#run restore method
if($method eq 'restore'){
	#verify we have a good index
	$goodindex = `$nsdchat ArchiveIndex names | grep $index`;
	if($goodindex eq ''){
		$date = `date`;
		$date =~ s/\n//;
		print LOGFILE "$date -- Could not find Archive Index \"$index\", exiting.  Please check the \$index variable in aw-queue.pl\n";
		exit;
	}
	
	#create job
	$restore_selection = `$nsdchat RestoreSelection create $client`;
	$restore_selection =~ s/\n//;

	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Restore selection \"$restore_selection\" created successfully\n";

	
	#open queue file, add each file to restore queue
	open FILE, "<",$full_queue_path or die $!;
	$i = 0;
	while (<FILE>){	
		$_ =~ s/\n//;
		if (defined($_)){

			$date = `date`;
			$date =~ s/\n//;
			print LOGFILE "$date -- $nsdchat ArchiveEntry handle $client {$_} $index\n";
			@archive_handles[$i] = `$nsdchat ArchiveEntry handle $client {$_} $index`;	
			chomp(@archive_handles[$i]);
			print LOGFILE "$date -- $nsdchat RestoreSelection $restore_selection addentry {@archive_handles[$i]}\n";			
			@restore_handles[$i] = `$nsdchat RestoreSelection $restore_selection addentry {@archive_handles[$i]}`;
			
			$date = `date`;
			$date =~ s/\n//;
			print LOGFILE "$date -- File $_ returned archive handle successfully\n";		
			print LOGFILE "$date -- Archive Handle $_ generated restore handle successfully\n";
			
		} else {
			$date = `date`;
			$date =~ s/\n//;
		
			print LOGFILE "$date -- File $_ could not be found in index $index.\n";
		}
		$i++;
	}
	
	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Preparing $i files for restore\n";		
	
	#submit restore job to run

	$job_id = `$nsdchat RestoreSelection $restore_selection submit`;
	$job_id =~ s/\n//;
	
	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Restore job $job_id successfully submitted\n";
	
	open FILE,">",$full_queue_path;
	close FILE;
	
	$date = `date`;
	$date =~ s/\n//;
	print LOGFILE "$date -- Restore queue emptied\n";
	
}

$date = `date`;
$date =~ s/\n//;
print LOGFILE "$date -- Script Exited Cleanly\n";

close LOGFILE;